@@ -13,7 +13,7 @@ except ImportError:
 class TestCodeFormat(unittest.TestCase):
     def test_conformance(self):
         """Test that we conform to PEP-8."""
-        style = pycodestyle.StyleGuide(quiet=False, ignore=['E501', 'W605'])
+        style = pycodestyle.StyleGuide(quiet=False, ignore=['E501', 'W605', 'W503'])
         if exists('transitions'):  # when run from root directory (e.g. tox)
             style.input_dir('transitions')
             style.input_dir('tests')
