@@ -1233,22 +1233,31 @@ class TestTransitions(TestCase):
         m.add_transition('stop', 'B', 'C')
         m.add_transition('wait', 'B', None)
         assert d.may_walk()
+        assert d.may_trigger("walk")
         assert not d.may_stop()
+        assert not d.may_trigger("stop")
         assert not d.may_wait()
+        assert not d.may_trigger("wait")
 
         d.walk()
         assert not d.may_walk()
+        assert not d.may_trigger("walk")
         assert d.may_stop()
+        assert d.may_trigger("stop")
         assert d.may_wait()
+        assert d.may_trigger("wait")
 
     def test_may_transition_for_autogenerated_triggers(self):
         states = ['A', 'B', 'C']
         m = Machine(states=states, initial='A')
         assert m.may_to_A()
+        assert m.may_trigger("to_A")
         m.to_A()
         assert m.to_B()
+        assert m.may_trigger("to_B")
         m.to_B()
         assert m.may_to_C()
+        assert m.may_trigger("to_C")
         m.to_C()
 
     def test_may_transition_with_conditions(self):
@@ -1259,18 +1268,14 @@ class TestTransitions(TestCase):
         m.add_transition('stop', 'B', 'C')
         m.add_transition('run', 'A', 'C')
         assert not d.may_walk()
+        assert not d.may_trigger("walk")
         assert not d.may_stop()
+        assert not d.may_trigger("stop")
         assert d.may_run()
+        assert d.may_trigger("run")
         d.run()
         assert not d.may_run()
-
-    def test_may_transition_with_auto_transitions(self):
-        states = ['A', 'B', 'C']
-        d = DummyModel()
-        self.machine_cls(model=d, states=states, initial='A')
-        assert d.may_to_A()
-        assert d.may_to_B()
-        assert d.may_to_C()
+        assert not d.may_trigger("run")
 
     def test_machine_may_transitions(self):
         states = ['A', 'B', 'C']
@@ -1281,12 +1286,18 @@ class TestTransitions(TestCase):
         m.add_transition('reset', 'C', 'A')
 
         assert not m.may_walk()
+        assert not m.may_trigger("walk")
         assert not m.may_stop()
+        assert not m.may_trigger("stop")
         assert m.may_run()
+        assert m.may_trigger("run")
         m.run()
         assert not m.may_run()
+        assert not m.may_trigger("run")
         assert not m.may_stop()
+        assert not m.may_trigger("stop")
         assert not m.may_walk()
+        assert not m.may_trigger("walk")
 
     def test_may_transition_with_invalid_state(self):
         states = ['A', 'B', 'C']
@@ -1294,6 +1305,7 @@ class TestTransitions(TestCase):
         m = self.machine_cls(model=d, states=states, initial='A', auto_transitions=False)
         m.add_transition('walk', 'A', 'UNKNOWN')
         assert not d.may_walk()
+        assert not d.may_trigger("walk")
 
     def test_may_transition_with_exception(self):
         stuff = Stuff(machine_cls=self.machine_cls, extra_kwargs={"send_event": True})
@@ -1314,11 +1326,15 @@ class TestTransitions(TestCase):
         assert stuff.works()
         with self.assertRaises(ValueError):
             stuff.may_raises()
+        with self.assertRaises(ValueError):
+            stuff.may_trigger("raises")
         assert stuff.is_B()
         stuff.machine.on_exception.append(process_exception)
         assert not stuff.may_raises()
+        assert not stuff.may_trigger("raises")
         assert stuff.to_A()
         assert not stuff.may_raises()
+        assert not stuff.may_trigger("raises")
 
     def test_on_final(self):
         final_mock = MagicMock()
