@@ -395,8 +395,8 @@ class HierarchicalMachine(Machine):
                  send_event=False, auto_transitions=True,
                  ordered_transitions=False, ignore_invalid_triggers=None,
                  before_state_change=None, after_state_change=None, name=None,
-                 queued=False, prepare_event=None, finalize_event=None, model_attribute='state', on_exception=None,
-                 **kwargs):
+                 queued=False, prepare_event=None, finalize_event=None, model_attribute='state',
+                 model_override=False, on_exception=None, on_final=None, **kwargs):
         assert issubclass(self.state_cls, NestedState)
         assert issubclass(self.event_cls, NestedEvent)
         assert issubclass(self.transition_cls, NestedTransition)
@@ -410,7 +410,7 @@ class HierarchicalMachine(Machine):
             ordered_transitions=ordered_transitions, ignore_invalid_triggers=ignore_invalid_triggers,
             before_state_change=before_state_change, after_state_change=after_state_change, name=name,
             queued=queued, prepare_event=prepare_event, finalize_event=finalize_event, model_attribute=model_attribute,
-            on_exception=on_exception, **kwargs
+            model_override=model_override, on_exception=on_exception, on_final=on_final, **kwargs
         )
 
     def __call__(self, to_scope=None):
@@ -759,9 +759,11 @@ class HierarchicalMachine(Machine):
     def _can_trigger(self, model, trigger, *args, **kwargs):
         state_tree = self.build_state_tree(getattr(model, self.model_attribute), self.state_cls.separator)
         ordered_states = resolve_order(state_tree)
-        for state_path in ordered_states:
-            with self():
-                return self._can_trigger_nested(model, trigger, state_path, *args, **kwargs)
+        with self():
+            return any(
+                self._can_trigger_nested(model, trigger, state_path, *args, **kwargs)
+                for state_path in ordered_states
+            )
 
     def _can_trigger_nested(self, model, trigger, path, *args, **kwargs):
         if trigger in self.events:
@@ -822,14 +824,15 @@ class HierarchicalMachine(Machine):
         return trigger in state.events or any(self.has_trigger(trigger, sta) for sta in state.states.values())
 
     def is_state(self, state, model, allow_substates=False):
-        if allow_substates:
-            current = getattr(model, self.model_attribute)
-            current_name = self.state_cls.separator.join(self._get_enum_path(current))\
-                if isinstance(current, Enum) else current
-            state_name = self.state_cls.separator.join(self._get_enum_path(state))\
-                if isinstance(state, Enum) else state
-            return current_name.startswith(state_name)
-        return getattr(model, self.model_attribute) == state
+        tree = self.build_state_tree(listify(getattr(model, self.model_attribute)),
+                                     self.state_cls.separator)
+
+        path = self._get_enum_path(state) if isinstance(state, Enum) else state.split(self.state_cls.separator)
+        for elem in path:
+            if elem not in tree:
+                return False
+            tree = tree[elem]
+        return len(tree) == 0 or allow_substates
 
     def on_enter(self, state_name, callback):
         """Helper function to add callbacks to states in case a custom state separator is used.
@@ -946,8 +949,7 @@ class HierarchicalMachine(Machine):
                 self._checked_assignment(model, 'is_' + path[0], FunctionWrapper(trig_func))
         with self(state.name):
             for event in self.events.values():
-                if not hasattr(model, event.name):
-                    self._add_trigger_to_model(event.name, model)
+                self._add_trigger_to_model(event.name, model)
             for a_state in self.states.values():
                 self._add_model_to_state(a_state, model)
 
@@ -996,10 +998,16 @@ class HierarchicalMachine(Machine):
         self._init_state(new_state)
 
     def _add_machine_states(self, state, remap):
-        new_states = [s for s in state.states.values() if remap is None or s not in remap]
+        new_states = [s for s in state.states.values() if remap is None or (s.name if hasattr(s, "name") else s) not in remap]
         self.add_states(new_states)
         for evt in state.events.values():
-            self.events[evt.name] = evt
+            # skip auto transitions
+            if state.auto_transitions and evt.name.startswith('to_') and evt.name[3:] in state.states:
+                continue
+            if evt.transitions and evt.name not in self.events:
+                self.events[evt.name] = evt
+                for model in self.models:
+                    self._add_trigger_to_model(evt.name, model)
         if self.scoped.initial is None:
             self.scoped.initial = state.initial
 
